/* AMXMOD natives
*
* (c) Copyright 2000-2002, OLO
* This file is provided as is (no warranties).
*/

#if defined _amxmod_included
#endinput
#endif
#define _amxmod_included

#include <amxconst>
#include <core>
#include <float>

/* Register plugin with some info
*
*/
native register_plugin(name[],version[],author[]);

/* This is required function for every script
* Just after plugin load this function is called
* (this is server activation time - also after map change)
*/
forward plugin_init();

/* Place that function in your script whenever you want 
* to catch moment when player changing his setinfo
* Index var is a index of player
*/
forward client_infochanged(id);

/* Place that function in your script whenever you want 
* to catch moment when player is connecting to server
*/
forward client_connect(id);

/* Place that function in your script whenever you want 
* to catch moment when player is disconnecting from server
*/
forward client_disconnect(id);

/* Place that function in your script whenever you want 
* to catch moment when player send the client command
* (it is recommended to use register_clcmd instead)
* You may return here PLUGIN_HANDLED to stop command from sending
* Function must return PLUGIN_CONTINUE or PLUGIN_HANDLED
*/
forward client_command(id);

/* Place that function in your script whenever you want 
* to catch moment when player is entering to the game
*/
forward client_putinserver(id);

/* This function is used to prechase any iteam
* Use here model_precache for that.
*/
forward plugin_precache();


/* Read line from file
* Function returns index of next line and 0 when end of file is reached
*
*/
native read_file(file[],line,text[],iLen);

/* Write line to file
* Function returns 0 when failure occurs
* When line is set to -1 the line is added at the end of file
*
*/
native write_file(file[],text[],line = -1);

/* Delete file. Function returns 1 on success, 0 on failure.
*
*/
native delete_file(file[]);

/* Check for file. If file exists function returns 1 in other case 0.
*
*/
native file_exists(file[]);

/* Precache model. Can be used only in plugin_precache function.
*
*/
native precache_model(name[]);

/* Precache sound. Can be used only in plugin_precache function.
*
*/
native precache_sound(name[]);

/* Set setinfo for player
*
*/
native set_user_info(player,name[],value[]);

/* Get setinfo from player
*
*/
native get_user_info(player,name[],outpu[],iLen);

/* Set localinfo for server
*
*/
native set_localinfo(name[],value[]);

/* Get localinfo from server
*
*/
native get_localinfo(name[],outpu[],iLen);

/* Show text in motd window
* When there is no header the motd title will be name of server
*/
native show_motd(player,message[],header[]="");

/* Send message to player
* types: print_notify - 1, print_console - 2, print_chat - 3, print_center - 4
* Message is limited to 180 chars (Center messages are limited to 70 chars)
* (set player to 0 for globals send)
* You may use here formating (f.e. client_print(1,print_chat,"[AMX] This is your %d. warning",2);
*/
native client_print(player,type,message[],{Float,_}:...);

/* Send message to player by engine
* types: engprint_console - 0, engprint_center - 1, engprint_chat - 2
* Message is limited to 180 chars (Center messages are limited to 70 chars)
* (set player to 0 for globals send)
* You may use here formating (f.e. engclient_print(1,engprint_chat,"[AMX] This is your %d. warning",2);
*/
native engclient_print(player,type,message[],{Float,_}:...);

/* Registers event on which the give function will be called
* flags:
* "a" - global event
* "b" - specified
* "c" - send only once when repeated to other players
* "d" - call if is send to dead player
* "e" - to alive
*
* Examples for conditions: 
* "2=c4" - 2nd parameter of message must be sting "c4"
* "3>10" - 3rd parameter must be greater then 10
* "3!4" - 3rd must be different from 4
* "2&Buy" - 2nd parameter of message must contain "Buy" substring
* "2!Buy" - 2nd parameter of message can't contain "Buy" substring
*
*/
native register_event(event[],function[],flags[],cond[]="", ... );

/* Sets format for hudmessage
* Change channel to display many messages at the same time
*
*/
native set_hudmessage(red=200, green=100, blue=0, Float:x=-1.0, Float:y=0.35, effects=0, Float:fxtime=6.0, Float:holdtime=12.0, Float:fadeintime=0.1, Float:fadeouttime=0.2,channel=4)

/* player is index (0 makes it gloabl)
* Message is auto splited so no limits for messages
*/
native show_hudmessage(player,message[],{Float,_}:...);

/* Displays menu, player is a index, keys are set as bits (1 = 1, 2 = 2, 3 = 4, 4 = 8 ... 0 = 512)
* When time is set to -1 then menu won't wanish until permiting the action
* (set player to 0 for globals send)
*/
native show_menu(player,keys,menu[], time=-1);

/* used to get values from engine messages
* when you are asking for string the char array and size var is needed (f.e. read_data(2,name[],iLen)
* floats and integers are returned by function (f.e. new me = read_data(3))
*/
native read_data(value, ... );

/* Prints output to server console
* You may use there formating (f.e. server_print("%s %d!","hello",7))
*
*/
native server_print(message[], {Float,_}:...);

/* Returns 1 or 0
*
*/
native is_map_valid(mapname[]);

/* Returns 1 or 0
*
*/
native is_user_bot(index);

/* Returns 1 or 0
*
*/
native is_user_connected(index);

/* Returns 1 or 0
*
*/
native is_user_alive(index);


/* Sets player godmode. If you want to disable godmode
* Just send func. with one parameter which is index of player.
*
*/
native set_user_godmode(index,godmode = 0);

/* Sets player frags
*
*/
native set_user_frags(index,frags);

/* Sets player armor
*
*/
native set_user_armor(index,armor);

/* Sets player health
*
*/
native set_user_health(index, health);

/* Returns 0 if player is not attacked by player
* (this is no attack or attack by object - worldspawn, grenade) 
* or in other case returns the index of player.
*
*/
native get_user_attacker(index);

/* if player is not hiting at anything function returns 0.0
* in other case the distance of aiming is returned.
* Dist is a distance on which we should check player aiming.
* Id keeps index on target and body will contain the part of body
* on which we aim.
*
*/
native Float:get_user_aiming(index,&id,&body,dist=9999);

/* Returns player frags
*
*/
native get_user_frags(index);

/* Returns player armor
*
*/
native get_user_armor(index);

/* Returns player health
*
*/
native get_user_health(index);

/* Returns index basing on name
*
*/
native get_user_index(name[]);

/* Returns ip
*
*/
native get_user_ip(index,ip[],iLen);

/* Coverts number from range 0 - 999 to words
*
*/
native num_to_word(num,output[],iLen);

/* Returns id of team. When function is passed
* with three parameters then the name of team is set
* on proper var.
*
*/
native get_user_team(index,team[]="",iLen=0);

/* Returns playing time in seconds
*
*/
native get_user_time(index);

/* Gets ping and loss at requested moment
*
*/
native get_user_ping(index, &ping, &loss);

/* Gets current position of player
*
*/
native get_user_origin(index, origin[3]);

/* Move player to the given position
*
*/
native set_user_origin(index, origin[3]);

/* Returns all carried weapons as bit sum
*
*/
native get_user_weapons(index);

/* Returns name of player
*
*/
native get_user_name(index,name[],iLen);

/* Returns authid (wonid) of player
*
*/
native get_user_authid(index,authid[],iLen);

/* Returns userid of player
*
*/
native get_user_userid(index)

/* Slaps pointed by index player with given power
*
*/
native user_slap(index,power);

/* Kills user pointed by index
* When the frag value is set to 1 then
* the kill won't decrase frags.
*
*/
native user_kill(index,frag=0);

/* Set player to glow.
* If you want to back to the default settings then set
* only first parameter (f.e. user_glow(index))
* Alpha sets the transparency of player.
* Values to set are from 0-255 range.
* Popular values are:
* fx = kRenderFxGlowShell and render = kRenderTransAlpha
*/
native user_glow(index,fx = kRenderFxNone, r=255,g=255,b=255, render = kRenderNormal,amount=16);

/* Send log message.
*
*/
native log_message(message[],{Float,_}:...);

/*   Returns number of players in game
*
*/
native get_playersnum();

/* Returns indexes of player is array of 32 players
* flags:
* "a" - don't collect dead players, 
* "b" - don't collect alive players, 
* "c" - skip bots, 
* "d" - skip real players, 
* "e" - match with team,
* For example we want all alive CT: get_players("ae",players[32],iNum,"CT");
*/
native get_players(players[32], &iNum,flags[]="", team[]="");

/* Use this native in client_command function
* id sets index for command parameter, arg and iLen are the output descriptors
* Function returns length of argument
*/
native read_argv(id,arg[],iLen);

/* Use this native in client_command function
*  returns line with all paramaters from the command line
*  Function returns length of arguments line
*/
native read_args(argline[],iLen);

/* Returns number of paramaters
*
*/
native read_argc();

/* Returns flags as number
* For example converts "abc" to bit sum 7
*/
native read_flags(flags[])

/*
* Find player basing on set flags
* flags:
* "a" - with given name
* "b" - with given part of name
* "c" - with given authid
* "d" - with given ip
* "e" - with given team name
* "f" - don't look in dead players
* "g" - don't look in alive players
* "h" - skip bots
* "i" - skip real players
* "j" - return last found match (not first as default)
*/
native find_player(flags[], name[]="" );

/* Removes quotes on begin and end from sentences like: "Hello World"
*
*/
native remove_quotes(text[])

/* Executes command on player (0 affects all)
*
*/
native client_cmd(index,command[],{Float,_}:...);

/* This is an emulation of client command (command is not send to client!)
* It allows to execute some commands on players and bots! (0 affects all)
* Function is excellent for forcing to do action related to game (not settings!)
* like "say hello world", "weapon_knife" or "chooseteam", "menuselect" "5"
* The command must stand alone but in arguments you can use spaces
*/
native engclient_cmd(index,command[],arg1[]="",arg2[]="");

/* Executes command on server
*
*/
native server_cmd(command[],{Float,_}:...);

/* Set cvar to given value
*
*/
native set_cvar_string(cvarname[],value[]);

/* If cvar exists return 1 in other case 0
*
*/
native cvar_exists(cvarname[])

/* Set cvar to given float
*
*/
native set_cvar_float(cvarname[],Float:value)


/* Get cvar float
*
*/
native Float:get_cvar_float(cvarname[])

/* Reads cvar's vaue
*
*/
native get_cvar_string(cvarname[],output[],iLen);

/* Returns name of currently played map
*
*/
native get_mapname(name[],iLen);

/* Returns time left in seconds
*
*/
native get_timeleft();

/* Returns seconds from map start
*
*/
native Float:get_gametime();

/* Returns name of currently played mod
*
*/
native get_modname(name[],iLen);

/* Returns time in given format
* The most popular is: "%m/%d/%Y - %H:%M:%S"
* But you may also request year: "%Y" or short weekday: "%a"
* See time.h for more details
*/
native get_time(format[],output[],iLen);

/* Set function to execute on specified time
* time is send as string, try to use unique ids so you will be able
* to better manage with remove_task, parm is a string with command line
* send to the function
* flags: a - repeat, b - loop task
*
*/
native set_task(Float:time,function[],id = 0,parm[]="",flags[]="", repeat = 0)

/* Remove task basing on id set on task creation
*
*/
native remove_task(id);

/* Read flags from player (server)
* Set index to 0 if you wan't to read flags in server
* Id is from range 0-31 to set flags on diff. vars
*/
native get_user_flags(index,id=0);

/* Remove flags for player (server)
* Set flags to -1 if you want to clear flags
* If you want to pass there "abc" format use read_flags function
*/
native remove_user_flags(index,flags,id=0);

/* Set flags for player (server)
* Set flags to -1 if you want to clear flags
* If you want to pass there "abc" format use read_flags function
*/
native set_user_flags(index,flags,id=0);

/* Set function call on specified client command
*
*/
native register_clcmd(clcmd[],function[]);

/* Set function call on specified server command
*
*/
native register_srvcmd(srvcmd[],function[]);

/* Register menu id. Function returns id of registered menu
* So you will be able to check by get_user_menu what
* menu player is currently watching. The name can be part
* of menu body.
*/
native register_menuid(name[]);

/* Register menu cmd which function will be executed
* when on proper menu the proper key is pressed.
* When function is called the parameters are:
* player index and pressed key (as bit)
*/
native register_menucmd(menuid,keys,function[]);

/* Sets what menu the player is watching and what keys for menu
* he had set. When there is no menu the index is 0. 
* If the value is negative index then the menu
* is VGUI in other case the id is from register_menu function.
*/
native get_user_menu(index,&id,&keys);

/* Forces server to execute sent server command now
* Very useful for map changes, setting cvars and other activities
*/
native server_exec();

/* Returns converted string to num
*
*/
native str_to_num(a[]);

/* Checks if two strings equal
* If c var is set then there are only c chars comapred
*/
native equal(a[],b[],c=0);

/* Copies one string to another
* Last var if diffrent from 0, specifies how many
* chars we want to copy.
* Function returns number of copied chars.
*/
native copy(dest[],iLen,src[],max=0);

/* Reads text and gets from that parameters to given arguments
* For example text: "^"This is^" the best year"
* You should pass through function parse(text,one,32,two,32,three,32,four,32)
* And you will get in one: "This is" in two "the" in three "best" and in four "year"
* Function returns number of read parameters
*
*/
native parse(text[], ... );


/* Emit sound. At the end of this file are some constans for this function.
* You should search for examples in the HL SDK.
*
*/
native emit_sound(index, channel, sample[], Float:vol, Float:att,flags, pitch);

/* Checks if src contains str. On success function
* returns position in src, on failure returns -1.
*
*/
native contain(src[],str[]);

/* Add one string to another
* Last var if diffrent from 0, specifies how many
* chars we want to add.
* Function returns number of all merged chars
*/
native add(dest[],iLen,src[],max=0);

/* Copies one string to another with format
* %s for string; %d, %i, %x and %o for numbers; %c for chars
* (f.e. format(dest,"Hello %s. To day is %dth","World",5))
* Function returns number of copied chars.
*/
native format(dest[] ,iLen ,format[] , {Float,_}:...);

/* Converted number to string
*
*/
native num_to_str(a,b[],c);

/* Returns distance between two vectors
*
*/
native get_distance(origin1[3],origin2[3])

/* Register new cvar for HL engine
* After that it is a good to set values by set_cvar_ functions
* because registered cvar is initialized to 0
*/
native register_cvar(name[],string[],flags = 0,Float:float = 0.0);

/* Generate random floating point number from a to b
*
*/
native random_float(Float:a,Float:b);

/* Generate random integer from a to b
*
*/
native random_num(a,b);

/*
 * Pause function or plugin so it won't be called
 * flags:
 * a - pause whole plugin
 * b - pause function
 * c - look outside the plugin (by given plugin name)
 * when flag "a" is set then name of function is not needed
 * also when flag "c" is not set then the name can be null
 */
native pause(flag[], func[]="",plugin[]="");

/*
 * Unpause function or plugin
 * flags:
 * a - unpause whole plugin
 * b - unpause function
 * c - look outside the plugin (by given plugin name)
 * when flag "a" is set then name of function is not needed
 * also when flag "c" is not set then the name can be null
 */
native unpause(flag[], func[]="",plugin[]="");

/* Returns the id for engine message type
* (f.e. get_user_msgid("TextMsg"), get_user_msgid("DeathMsg"))
*/
native get_user_msgid(name[]);

/* These functinos are used to generate enigne messages
* For example you may generate menu, smoke, shockwaves,
* thunderlights, intermission (endmap) and many many others
* See HLSDK for more examples
*
*/
native message_begin( dest, msg_type, origin[3]={0,0,0},player=0);
native message_end();
native write_byte( x );
native write_char( x );
native write_short( x );
native write_long( x );
native write_entity( x );
native write_angle( x );
native write_coord( x );
native write_string( x[] );

